package org.aohe.core.utils;

import cn.hutool.core.net.Ipv4Util;
import cn.hutool.core.net.NetUtil;
import org.aohe.core.sane.utils.SaneSessionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.*;
import java.util.Enumeration;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class SystemUtils {

    private static final Logger log = LoggerFactory.getLogger(SystemUtils.class);

    /**
     * 判断操作系统是否是 Windows
     *
     * @return true：操作系统是 Windows
     *         false：其它操作系统
     */
    public static boolean isWindows() {
        String osName = getOsName();
        
        return osName != null && osName.startsWith("Windows");
    }
    
    /**
     * 判断操作系统是否是 MacOS
     *
     * @return true：操作系统是 MacOS
     *         false：其它操作系统
     */
    public static boolean isMacOs() {
        String osName = getOsName();
        
        return osName != null && osName.startsWith("Mac");
    }
    
    /**
     * 判断操作系统是否是 Linux
     *
     * @return true：操作系统是 Linux
     *         false：其它操作系统
     */
    public static boolean isLinux() {
        String osName = getOsName();
        
        return (osName != null && osName.startsWith("Linux")) || (!isWindows() && !isMacOs());
    }
    
    /**
     * 获取操作系统名称
     * @return os.name 属性值
     */
    public static String getOsName() {
        return System.getProperty("os.name");
    }

    /**
     *
     * @return
     */
    public static String getIP(){
        try {
            InetAddress addr = Inet4Address.getLocalHost();
            NetworkInterface network = NetworkInterface.getByInetAddress(addr);
            log.info(addr.getHostAddress());
            short mask = network.getInterfaceAddresses().get(0).getNetworkPrefixLength();
            //Ipv4Util.list(addr, mask);
            log.info(mask + "" );
        } catch (UnknownHostException | SocketException e) {
            throw new RuntimeException(e);
        }
        return "";
    }

    /**
     * 获取第一个内网网卡地址和掩码
     * @return 地址
     */
    public static String getInnerFirstEthMask(){
        Enumeration<NetworkInterface> netInterfaces;
        try {
            // 拿到所有网卡
            netInterfaces = NetworkInterface.getNetworkInterfaces();
            InetAddress ip;
            // 遍历每个网卡，拿到 ip
            while (netInterfaces.hasMoreElements()) {
                NetworkInterface ni = netInterfaces.nextElement();
                Enumeration<InetAddress> addresses = ni.getInetAddresses();
                while (addresses.hasMoreElements()) {
                    ip = addresses.nextElement();
                    if (!ip.isLoopbackAddress() && ip.getHostAddress().indexOf(':') == -1) {
                        if(NetUtil.isInnerIP(ip.getHostAddress())){
                            //
                            if(ni.getName().startsWith("eth")){
                                NetworkInterface network = NetworkInterface.getByInetAddress(ip);
                                short mask = 24;
                                for(InterfaceAddress a : network.getInterfaceAddresses()){
                                    if(ip.equals(a.getAddress())){
                                        //遍历所有 取ip位相同的mask
                                        mask = a.getNetworkPrefixLength();
                                    }
                                }
                                System.out.println(mask);
                                return ip.getHostAddress()+"/"+mask;
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取网卡信息失败");
        }
        return "127.0.0.1/24";
    }

}